==> lhcsmapi-1.3.229.tar.gz: lhcsmapi-1.3.229/lhcsmapi/gui/pc/RbFgcPmSearchBaseModule.py <==
from lhcsmapi.gui.pc.FgcPmSearchBaseModule import FgcPmSearchBaseModule
from lhcsmapi.dbsignal.post_mortem.PmDbRequest import PmDbRequest
from lhcsmapi.Time import Time
from lhcsmapi.metadata.SignalMetadata import SignalMetadata


class RbFgcPmSearchBaseModule(FgcPmSearchBaseModule):

    def get_circuit_names(self, index, fgc_pm_entries):
        return fgc_pm_entries[index][0]

    def get_circuit_timestamps(self, index, fgc_pm_entries):
        return fgc_pm_entries[index][1]

    def convert_fgc_pm_entries_to_options(self, fgc_pm_entries):
        return [Time.to_string(el[1]) for el in fgc_pm_entries]

    def find_fgc_pm_entries_with_qps(self, start_time, end_time, circuit_name):
        # ToDo: Split into two methods as for RQ
        metadata_fgc = SignalMetadata.get_circuit_signal_database_metadata('RB', circuit_name, 'PC', 'PM')

        quotient, remainder = Time.modulo_day_in_unix_timestamp(start_time, end_time)
        event_duration_nsec = int(24 * 60 * 60 * 1e9)
        source_timestamp_fgc = []

        self.progress_search.layout.visibility = 'visible'
        self.progress_search.max = quotient
        self.progress_search.description = f'Days: 0/{quotient}'

        for day in range(quotient):
            start_time_temp = Time.to_unix_timestamp(start_time) + int(day * event_duration_nsec)

            # Update GUI
            self.lbl_search.value = 'Querying PM for FGC entries on {}'.format(Time.to_string(start_time_temp))

            source_timestamp_fgc += PmDbRequest.find_events(metadata_fgc['source'], metadata_fgc['system'],
                                                            metadata_fgc['className'], t_start=start_time_temp,
                                                            duration=[(event_duration_nsec, 'ns')])
            self.progress_search.description = f'Days: {day + 1}/{quotient}'
            self.progress_search.value = day + 1

        if remainder > 0:
            start_time_temp = Time.to_unix_timestamp(end_time) - remainder
            self.lbl_search.value = 'Querying PM for FGC entries on {}'.format(Time.to_string(start_time_temp))
            source_timestamp_fgc += PmDbRequest.find_events(metadata_fgc['source'], metadata_fgc['system'],
                                                            metadata_fgc['className'], t_start=start_time_temp,
                                                            duration=[(remainder, 'ns')])
        # Check if an FGC timestamp has a QPS discharge around
        source_timestamp_fgc_with_qps = []
        metadata_qds = SignalMetadata.get_circuit_signal_database_metadata('RB', circuit_name, 'QDS', 'PM')

        for index, elem in enumerate(source_timestamp_fgc):
            # Update GUI
            self.lbl_search.value = 'Checking if there are QPS PM around an FGC timestamp {}'.format(
                Time.to_string(elem[1]))
            self.progress_search.description = f'FGC: {index + 1}/{len(source_timestamp_fgc)}'
            self.progress_search.value = index + 1
            source_timestamp_qds = PmDbRequest.find_events('*', metadata_qds['system'], metadata_qds['className'],
                                                           t_start=elem[1], duration=[(60, 's'), (300, 's')])
            source_timestamp_qds_filtered = list(
                filter(lambda x: SignalMetadata.get_sector_names_from_components('RB', 'MAGNET', x[0]) in
                                 circuit_name, source_timestamp_qds))

            if source_timestamp_qds_filtered:
                source_timestamp_fgc_with_qps.append(elem)

        self.lbl_search.value = 'Query completed.'

        return source_timestamp_fgc_with_qps

==> lhcsmapi-1.4.58.tar.gz: lhcsmapi-1.4.58/lhcsmapi/gui/pc/RbFgcPmSearchBaseModule.py <==
from typing import List, Tuple

from lhcsmapi.gui.pc.FgcPmSearchBaseModule import FgcPmSearchBaseModule
from lhcsmapi.pyedsl.dbsignal.post_mortem.PmDbRequest import PmDbRequest
from lhcsmapi.Time import Time
from lhcsmapi.metadata.SignalMetadata import SignalMetadata


class RbFgcPmSearchBaseModule(FgcPmSearchBaseModule):
    """ A class with methods to search for RB FGC PM events to fill-in a search field in the FGC PM search module.

    """

    def convert_fgc_pm_entries_to_options(self, fgc_pm_entries: List[Tuple[str, int]]) -> List[str]:
        """ Method converting a list of source and timestamp tuples into a list of strings to be displayed

        :param fgc_pm_entries: a list of FGC PM source and timestamp tuples
        :return: list of strings with FGC PM source and timestamp
        """
        return [Time.to_string(el[1]) for el in fgc_pm_entries]

    def find_fgc_pm_entries_with_qps(self, start_time, end_time, circuit_name: str) -> List[Tuple[str, int]]:
        """ Method finding FGC PM entries with QPS for a given time range and circuit name

        :param start_time: start time for the query
        :param end_time: end time for the query
        :param circuit_name: circuit name
        :return: a list of FGC source and timestamp tuples
        """
        metadata_fgc = SignalMetadata.get_circuit_signal_database_metadata('RB', circuit_name, 'PC', 'PM')

        quotient, remainder = Time.modulo_day_in_unix_timestamp(start_time, end_time)
        event_duration_nsec = int(24 * 60 * 60 * 1e9)
        source_timestamp_fgc = []

        self.progress_search.layout.visibility = 'visible'
        self.progress_search.max = quotient
        self.progress_search.description = f'Days: 0/{quotient}'

        for day in range(quotient):
            start_time_temp = Time.to_unix_timestamp(start_time) + int(day * event_duration_nsec)

            # Update GUI
            self.lbl_search.value = 'Querying PM for FGC entries on {}'.format(Time.to_string(start_time_temp))

            source_timestamp_fgc += PmDbRequest.find_events(metadata_fgc['source'], metadata_fgc['system'],
                                                            metadata_fgc['className'], t_start=start_time_temp,
                                                            duration=[(event_duration_nsec, 'ns')])
            self.progress_search.description = f'Days: {day + 1}/{quotient}'
            self.progress_search.value = day + 1

        if remainder > 0:
            start_time_temp = Time.to_unix_timestamp(end_time) - remainder
            self.lbl_search.value = 'Querying PM for FGC entries on {}'.format(Time.to_string(start_time_temp))
            source_timestamp_fgc += PmDbRequest.find_events(metadata_fgc['source'], metadata_fgc['system'],
                                                            metadata_fgc['className'], t_start=start_time_temp,
                                                            duration=[(remainder, 'ns')])
        # Check if an FGC timestamp has a QPS discharge around
        source_timestamp_fgc_with_qps = self.check_quench_presence_around_fqc_timestamp(circuit_name,
                                                                                        source_timestamp_fgc)

        return source_timestamp_fgc_with_qps

    def check_quench_presence_around_fqc_timestamp(self, circuit_name: str, source_timestamp_fgc: List[Tuple[str, int]]
                                                   ) -> List[Tuple[str, int]]:
        """ Method checking whether there is a quench around FGC PM events and keeping only those events

        :param circuit_name: circuit name
        :param source_timestamp_fgc: a list of FGC PM source and timestamp tuples
        :return: a list of FGC PM source and timestamp tuples in vicinity of quenches
        """
        source_timestamp_fgc_with_qps = []
        metadata_qds = SignalMetadata.get_circuit_signal_database_metadata('RB', circuit_name, 'QDS', 'PM')
        for index, elem in enumerate(source_timestamp_fgc):
            # Update GUI
            self.lbl_search.value = 'Checking if there are QPS PM around an FGC timestamp {}' \
                .format(Time.to_string(elem[1]))
            self.progress_search.description = f'FGC: {index + 1}/{len(source_timestamp_fgc)}'
            self.progress_search.value = index + 1
            source_timestamp_qds = PmDbRequest.find_events('*', metadata_qds['system'], metadata_qds['className'],
                                                           t_start=elem[1], duration=[(60, 's'), (300, 's')])
            source_timestamp_qds_filtered = list(
                filter(lambda x: SignalMetadata.get_sector_names_from_components('RB', 'MAGNET', x[0]) in
                                 circuit_name, source_timestamp_qds))

            if source_timestamp_qds_filtered:
                source_timestamp_fgc_with_qps.append(elem)
        self.lbl_search.value = 'Query completed.'
        return source_timestamp_fgc_with_qps
